//
//  SZSimplePhysicsDragSelector.m
//  SimplePhysicsDragSelectorTest
//
//  Created by 朱笑月 on 13-12-12.
//
//

#import "SZSimplePhysicsDragSelector.h"

@implementation SZSimplePhysicsDragSelector

@synthesize Delegate = s_delegate;
@synthesize Icons = s_icons;
@synthesize SelectedIcon = s_selectedIcon;

- (id)initWithIcons:(NSArray *)icons
{
    self = [super init];
    if (self) {
        s_icons = [[NSMutableArray alloc] initWithArray:icons];
        s_content = nil;
        s_selectedIcon = nil;
        
        isDragging = NO;
        lastTouchPoint = CGPointZero;
		lastx = 0.0f;
		xvel = 0.0f;
		minX = 0;
        maxX = 0;
        acceleration = 0.0f;
        f = 0.0f;
        
        self.isTouchEnabled = true;
        
        s_delegate = nil;
    }
    return self;
}

- (void)dealloc
{
    [s_icons release];
    [super dealloc];
}

#pragma mark - Custom methods
-(void) layout
{
    int i = 1;
    for (CCNode *icon in s_icons) {
        CGPoint position = ccp((i-1) * 180, 0);
        float distance = fabsf(icon.position.x)/100;
        float scale = 1/(1+distance);
        icon.position = position;
        icon.scale = scale;
        if (![s_content.children containsObject:icon]) {
            [s_content addChild:icon];
        }
        i++;
    }
    s_selectedIcon = [s_icons lastObject];
    if ([s_delegate respondsToSelector:@selector(onSelectedIconChanged:)]) {
        [s_delegate onSelectedIconChanged:self];
    }
    
    minX = - (i-1) * 180 - 100;
    maxX = 100;
}

- (void) updateMove:(ccTime)dt
{
	if ( !isDragging )
	{
        // *** CHANGE BEHAVIOR HERE *** //
        float F1 = 0.0f;
        float F2 = 0.0f;
        float F3 = 0.0f;
        CGPoint pos = s_content.position;
        
        //F1
        // friction
        F1 = - xvel * 0.1;
        
        //F2
        // prevent icons out of range
        if ( pos.x < minX )
        {
            F2 = (minX - pos.x);
        }
        else if ( pos.x > maxX )
        {
            F2 = (maxX - pos.x);
        }
        
        //F3
        // suck planet
        if (fabsf(xvel) < 100 && !s_selectedIcon) {
            CCNode *nearestIcon = nil;
            for (CCNode *icon in s_icons) {
                if (nearestIcon) {
                    CGPoint pt1 = [icon.parent convertToWorldSpace:icon.position];
                    float distance1 = fabsf(pt1.x - [CCDirector sharedDirector].winSize.width/2);
                    CGPoint pt2 = [nearestIcon.parent convertToWorldSpace:nearestIcon.position];
                    float distance2 = fabsf(pt2.x - [CCDirector sharedDirector].winSize.width/2);
                    if (distance1 < distance2) {
                        nearestIcon = icon;
                    }
                }
                else {
                    nearestIcon = icon;
                }
            }
            if (nearestIcon) {
                s_selectedIcon = nearestIcon;
                if ([s_delegate respondsToSelector:@selector(onSelectedIconChanged:)]) {
                    [s_delegate onSelectedIconChanged:self];
                }
            }
        }
        
        if (s_selectedIcon) {
            CGPoint pt = [s_selectedIcon.parent convertToWorldSpace:s_selectedIcon.position];;
            float distance = pt.x - [CCDirector sharedDirector].winSize.width/2;
            F3 = - distance;
        }
        
        //CALCULATE
        f = F1 + F2 + F3;
        acceleration = f/1;
        xvel += acceleration;
        pos.x += xvel*dt;
        
		s_content.position = pos;
	}
	else
	{
		xvel = ( s_content.position.x - lastx ) / dt;
		lastx = s_content.position.x;
	}
}

-(void) updateScale:(ccTime)dt;
{
    for (CCNode *icon in s_icons) {
        CGPoint pt = [self convertToNodeSpace:[icon.parent convertToWorldSpace:icon.position]];
        float distance = fabsf(pt.x)/100;
        icon.scale = 1/(1+distance);
    }
}

- (void)ccTouchesBegan:(NSSet *)touches withEvent:(UIEvent *)event;
{
    s_selectedIcon = nil;
    if ([s_delegate respondsToSelector:@selector(onSelectedIconChanged:)]) {
        [s_delegate onSelectedIconChanged:self];
    }
    UITouch *touch = [touches anyObject];
    CGPoint position = [self convertTouchToNodeSpace:touch];
    lastTouchPoint = position;
    isDragging = true;
}

- (void)ccTouchesMoved:(NSSet *)touches withEvent:(UIEvent *)event;
{
    UITouch *touch = [touches anyObject];
    CGPoint position = [self convertTouchToNodeSpace:touch];
    CGPoint translate = ccpSub(position, lastTouchPoint);
    translate.y = 0;
    s_content.position = ccpAdd(s_content.position, translate);
    lastTouchPoint = position;
}

- (void)ccTouchesEnded:(NSSet *)touches withEvent:(UIEvent *)event;
{
    isDragging = false;
    if (s_content.position.x > maxX) {
        if ([s_delegate respondsToSelector:@selector(onDragSelectorBackward:)]) {
            [s_delegate onDragSelectorBackward:self];
        }
    }
    if (s_content.position.x < minX) {
        if ([s_delegate respondsToSelector:@selector(onDragSelectorForward:)]) {
            [s_delegate onDragSelectorForward:self];
        }
    }
}

- (void)ccTouchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event;
{
    isDragging = false;
    if (s_content.position.x > maxX) {
        if ([s_delegate respondsToSelector:@selector(onDragSelectorBackward:)]) {
            [s_delegate onDragSelectorBackward:self];
        }
    }
    if (s_content.position.x < minX) {
        if ([s_delegate respondsToSelector:@selector(onDragSelectorForward:)]) {
            [s_delegate onDragSelectorForward:self];
        }
    }
}

-(void) update:(ccTime)dt;
{
    [self updateMove:dt];
    [self updateScale:dt];
}

#pragma mark Override methods

-(void) onEnter
{
    [super onEnter];
    s_content = [[CCSprite alloc]init];
    [self addChild:s_content];
    
    [self layout];
    
    [self scheduleUpdate];
}

-(void) onExit
{
    [self unscheduleUpdate];
    [self removeChild:s_content cleanup:YES];
    [s_content release];
    s_content = nil;
    
    s_selectedIcon = nil;
    
    [super onExit];
}

@end
